import random
from flask import Flask, render_template, request
import tictactoe

app = Flask(__name__)


ascii2board = lambda ascii: tuple(a if a != ' ' else None for a in ascii)


@app.route('/')
def index():
    return render_template('index.html')


@app.route('/play/start/<who>')
def start_game(who='player'):
    board, winner = tictactoe.EMPTY_BOARD, None
    if who == 'cpu':
        board, winner = tictactoe.play_random_move(board, 'O')

    return render_template('play.html', board=board, winner=winner)


@app.route('/play', methods=['POST'])
def play_game():
    board = ascii2board(request.form['board'])
    col, row = request.form['move'].split(',')
    col, row = int(col), int(row)
    board, winner = tictactoe.play(board, 'X', col, row)

    if winner is None:
        if random.random() < 0.1:
            board, winner = tictactoe.play_random_move(board, 'O')
        else:
            board, winner = tictactoe.play_best_move(board, 'O')

    return render_template('play.html', board=board, winner=winner)

@app.context_processor
def tictactoe_context_processor():
    def board_cell(board, col, row):
        return board[tictactoe.board_index(col, row)]

    return {'board_cell': board_cell}


@app.template_filter('board2ascii')
def board2ascii(board):
    return reduce(lambda a, b: a + (b if b is not None else ' '), board, '')


if __name__ == '__main__':
    app.run(debug=True)  # TODO remove this for production!
